/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: gui_draw.c 476 2006-08-01 06:26:05Z ed $
 */
/**
 * @file gui/gui_draw.c
 */

#include "config.h"

#include <curses.h>

#include <gui/gui.h>
#include <configfile/configfile.h>

pthread_mutex_t gui_lock;

#ifdef HAVE_CURSES_WITHOUT_SIGWINCH
#include <fcntl.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/ttycom.h>

static void
gui_draw_catch_resize(int sig)
{
	int ttydev;
	struct winsize ws;

	if ((ttydev = open("/dev/tty", O_RDONLY)) < 0)
		return;
	if (ioctl(ttydev, TIOCGWINSZ, &ws) == -1)
		return;
	
	GUI_LOCK;
	resizeterm(ws.ws_row, ws.ws_col);
	refresh();
	GUI_UNLOCK;

	gui_statbar_resize();
	gui_playq_resize();
	gui_browser_resize();
}
#endif

void
gui_draw_init(void)
{
	/* Lock playq from main thread */
	pthread_mutex_init(&gui_lock, NULL);

	initscr();
	start_color();
	nonl();
	cbreak();
	noecho();
	keypad(stdscr, TRUE);
	raw();
	curs_set(0);	/* Hide cursor */
	refresh();

	if (has_colors()) {
		init_pair(GUI_COLOR_BAR,
		    config_getopt_color("gui.color.bar.fg"),
		    config_getopt_color("gui.color.bar.bg"));
		init_pair(GUI_COLOR_BLOCK,
		    config_getopt_color("gui.color.block.fg"),
		    config_getopt_color("gui.color.block.bg"));
		init_pair(GUI_COLOR_SELECT,
		    config_getopt_color("gui.color.select.fg"),
		    config_getopt_color("gui.color.select.bg"));
		init_pair(GUI_COLOR_DESELECT,
		    config_getopt_color("gui.color.deselect.fg"),
		    config_getopt_color("gui.color.deselect.bg"));
	}

	gui_statbar_init();
	gui_playq_init();

#ifdef HAVE_CURSES_WITHOUT_SIGWINCH
	signal(SIGWINCH, gui_draw_catch_resize);
#endif
}

void
gui_draw_destroy(void)
{
	gui_statbar_destroy();
	gui_playq_destroy();
	gui_browser_destroy();

	endwin();
}

#ifdef HAVE_WRESIZE
void
gui_draw_resize(void)
{
	GUI_LOCK;
	refresh();
	GUI_UNLOCK;
	gui_statbar_resize();
	gui_playq_resize();
	gui_browser_resize();
}
#endif /* HAVE_WRESIZE */
