/*
 * Copyright (c) 2006 Ed Schouten <ed@fxq.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: semaphore.c 423 2006-07-22 14:16:05Z ed $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "missing/semaphore.h"

#ifdef HAVE_BROKEN_SEM_INIT
sem_t *sem_wrap_init(char *name, unsigned int value)
{
	char semname_buf[32];
	snprintf(semname_buf, sizeof semname_buf, "herrie.%d.%s",
	    getpid(), name);
	return sem_open(semname_buf, O_CREAT | O_EXCL, 0600, value);
}
#else /* !HAVE_BROKEN_SEM_INIT */
sem_t *sem_wrap_unnamed_init(unsigned int value)
{
	sem_t *ret;
	ret = malloc(sizeof(sem_t));

	if (ret == NULL)
		return NULL;
	
	if (sem_init(ret, 0, value) != 0) {
		free(ret);
		return (NULL);
	}
	
	return (ret);
}
#endif /* HAVE_BROKEN_SEM_INIT */

void sem_wrap_destroy(sem_t *sem)
{
#ifdef HAVE_BROKEN_SEM_INIT
	sem_close(sem);
#else /* !HAVE_BROKEN_SEM_INIT */
	sem_destroy(sem);
	free(sem);
#endif /* HAVE_BROKEN_SEM_INIT */
}
