/*
 * Copyright (c) 2006 Martijn van den Dungen <martijn@il.fontys.nl>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: vfs_m3u.c 471 2006-08-01 05:35:07Z ed $
 */
/**
 * @file vfs/modules/vfs_m3u.c
 */

#include <missing/missing.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <vfs/vfs.h>
#include <vfs/modules/vfs_m3u.h>

/**
 * @brief Check if the extension of the given filename matches /\.m3u$/.
 */
static int
vfs_m3u_check_ext(char *name)
{
	size_t len;

	len = strlen(name);
	if (len < 4)
		return (-1);
	
	return strcasecmp(".m3u", &name[len - 4]);
}

int
vfs_m3u_open(struct vfs_ent *ve, int isdir)
{
	/* In order to speed up the process, we only match *.m3u */
	if (isdir || vfs_m3u_check_ext(ve->name) != 0)
		return (-1);

	return (0);
}

int
vfs_m3u_populate(struct vfs_ent *ve)
{
	FILE *fp;
	struct vfs_ent *new;
	size_t len;
	char *ch;
	char *ln, *fn;

	if ((fp = fopen(ve->filename, "rt")) == NULL)
		return (-1);

	while ((ln = fgetln(fp, &len)) != NULL) {
		/* Lines starting with # contain unsupported data */
		if (ln[0] == '#')
			continue;

		/* Add EOL and convert to proper separator */
		ln[len - 1] = '\0';
#if PATH_SEP != '\\'
		for (ch = ln; *ch != '\0'; ch++) {
			if (*ch == '\\')
				*ch = PATH_SEP;
		}
#endif

		if ((fn = malloc(strlen(ve->filename) +
		    strlen(ln) + 1)) == NULL)
			continue;
		strcpy(fn, ve->filename);
		ch = strrchr(fn, PATH_SEP);
		ch[1] = '\0';
		strcat(fn, ln);

		new = vfs_open(fn);
		free(fn);

		if (new == NULL)
			continue;

		TAILQ_INSERT_TAIL(&ve->population, new, next);
	}

	return (0);
}
